pub(crate) struct CharacterAndClassAndTrieValue {
c_and_c: CharacterAndClass,
pub trie_val: u32,
}Expand description
This struct makes the handling of the upcoming buffer
easily so that trie lookups are done at most once. However,
when upcoming[0] is an undecomposed starter, we don’t
need the ccc yet, and when lookahead has already done the
trie lookups, we don’t need trie_value, as it is implied
by ccc.
Fields
c_and_c: CharacterAndClasstrie_val: u32Implementations
Sourceimpl CharacterAndClassAndTrieValue
impl CharacterAndClassAndTrieValue
pub fn new_with_non_decomposing_starter(c: char) -> Self
pub fn new_with_non_zero_ccc(c: char, ccc: CanonicalCombiningClass) -> Self
pub fn new_with_non_special_decomposition_trie_val( c: char, trie_val: u32, ) -> Self
pub fn new_with_trie_val(c: char, trie_val: u32) -> Self
pub fn decomposition_starts_with_non_starter(&self) -> bool
pub fn character(&self) -> char
fn ccc(&self) -> CanonicalCombiningClass